@@ -39,22 +39,6 @@ static GQuark toolitem_id = 0;
 
 #define PARENT_TYPE bonobo_ui_sync_get_type ()
 
-static GdkPixbuf *
-cmd_get_toolbar_image (GtkWidget    *widget,
-		       BonoboUINode *node,
-		       BonoboUINode *cmd_node)
-{
-	if (bonobo_ui_node_peek_attr (node, "pixtype"))
-		return bonobo_ui_util_xml_get_pixbuf (
-			widget, node, GTK_ICON_SIZE_LARGE_TOOLBAR);
-
-	if (bonobo_ui_node_peek_attr (cmd_node, "pixtype"))
-		return bonobo_ui_util_xml_get_pixbuf (
-			widget, cmd_node, GTK_ICON_SIZE_LARGE_TOOLBAR);
-
-	return NULL;
-}
-
 static BonoboUIToolbarControlDisplay
 decode_control_disp (const char *txt)
 {
@@ -95,7 +79,6 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	char *min_width;
 	char *behavior;
 	char **behavior_array;
-	GdkPixbuf *image;
 	gboolean priority;
 
 	/* FIXME: to debug control problem */
@@ -131,11 +114,24 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	
 	if (!type || !strcmp (type, "toggle")) {
 
-		image = cmd_get_toolbar_image (widget, node, cmd_node);
+		if (BONOBO_IS_UI_TOOLBAR_BUTTON_ITEM (widget) &&
+		    (bonobo_ui_node_peek_attr (node, "pixtype") ||
+		     bonobo_ui_node_peek_attr (cmd_node, "pixtype"))) {
+			GtkWidget *image;
+			BonoboUIToolbarButtonItem *button_item;
 
-		if (image) {
-			bonobo_ui_toolbar_button_item_set_image (
-				BONOBO_UI_TOOLBAR_BUTTON_ITEM (widget), image);
+			button_item = (BonoboUIToolbarButtonItem *) widget;
+
+			image = bonobo_ui_toolbar_button_item_get_image (button_item);
+			if (!image) {
+				image = gtk_image_new ();
+				bonobo_ui_toolbar_button_item_set_image (button_item, image);
+			}
+
+			bonobo_ui_util_xml_set_image (
+				GTK_IMAGE (image), node, cmd_node,
+				GTK_ICON_SIZE_LARGE_TOOLBAR);
+			gtk_widget_show (image);
 		}
 
 		if (label)
@@ -470,7 +466,10 @@ parse_look (const char *look)
 			return BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
 
 		if (!strcmp (look, "text"))
-			return BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT;	
+			return BONOBO_UI_TOOLBAR_STYLE_TEXT_ONLY;
+
+		if (!strcmp (look, "both_horiz"))
+			return BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT;
 	}
 
 	return bonobo_ui_preferences_get_toolbar_style ();
@@ -623,11 +622,6 @@ create_dockitem (BonoboUISyncToolbar *sync,
 	gboolean can_config = TRUE;
 	BonoboUIToolbar *toolbar;
 
-	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
-		if (!strcmp (prop, "detachable"))
-			force_detachable = TRUE;
-	}
-
 	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
 		behavior_array = g_strsplit (prop, ",", -1);
 	
@@ -656,7 +650,6 @@ create_dockitem (BonoboUISyncToolbar *sync,
 		dockname, beh));
 
 	bonobo_dock_item_set_shadow_type (item, GTK_SHADOW_OUT);
-	gtk_container_set_border_width (GTK_CONTAINER (item), 2);
 
 	if ((prop = bonobo_ui_node_peek_attr (node, "placement"))) {
 		if (!strcmp (prop, "top"))
@@ -690,6 +683,7 @@ create_dockitem (BonoboUISyncToolbar *sync,
 		
 	toolbar = BONOBO_UI_TOOLBAR (bonobo_ui_toolbar_new ());
 
+	gtk_container_set_border_width (GTK_CONTAINER (toolbar), 2);
 	gtk_container_add (GTK_CONTAINER (item),
 			   GTK_WIDGET (toolbar));
 	gtk_widget_show (GTK_WIDGET (toolbar));
@@ -723,7 +717,7 @@ impl_bonobo_ui_sync_toolbar_remove_root (BonoboUISync *sync,
 {
 	const char *name;
 
-	if ((name = bonobo_ui_node_get_attr (node, "name"))) {
+	if ((name = bonobo_ui_node_peek_attr (node, "name"))) {
 		BonoboDockItem *item;
 
 		item = get_dock_item (BONOBO_UI_SYNC_TOOLBAR (sync), name);
@@ -740,6 +734,7 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	const char *txt;
 	const char *dockname;
 	gboolean    tooltips;
+	gboolean    detachable;
 	BonoboDockItem *item;
 	BonoboUIToolbar *toolbar;
 	BonoboUIToolbarStyle look;
@@ -753,7 +748,14 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	if (!item)
 		item = create_dockitem (BONOBO_UI_SYNC_TOOLBAR (sync),
 					node, dockname);
-	
+
+	if ((txt = bonobo_ui_node_peek_attr (node, "behavior")) &&
+	    strstr (txt, "detachable"))
+		detachable = TRUE;
+	else
+		detachable = bonobo_ui_preferences_get_toolbar_detachable ();
+	bonobo_dock_item_set_locked (item, !detachable);
+
 	toolbar = BONOBO_UI_TOOLBAR (GTK_BIN (item)->child);
 
 	bonobo_ui_engine_stamp_root (sync->engine, node, GTK_WIDGET (toolbar));
@@ -821,6 +823,16 @@ impl_bonobo_ui_sync_toolbar_can_handle (BonoboUISync *sync,
 		node->name_id == toolitem_id);
 }
 
+GtkWidget *
+impl_bonobo_ui_sync_toolbar_wrap_widget (BonoboUISync *sync,
+					 GtkWidget    *custom_widget)
+{
+	if (!BONOBO_IS_UI_TOOLBAR_ITEM (custom_widget))
+		return bonobo_ui_toolbar_control_item_new_widget (custom_widget);
+	else
+		return custom_widget;
+}
+
 static void
 class_init (BonoboUISyncClass *sync_class)
 {
@@ -842,6 +854,8 @@ class_init (BonoboUISyncClass *sync_class)
 
 	sync_class->state_update  = impl_bonobo_ui_sync_toolbar_state_update;
 	sync_class->can_handle    = impl_bonobo_ui_sync_toolbar_can_handle;
+
+	sync_class->wrap_widget   = impl_bonobo_ui_sync_toolbar_wrap_widget;
 }
 
 GType
